import 'CoreLibs/animation.lua'
import 'CoreLibs/graphics.lua'

--- Returns the normalized vector as two values, plus the distance between the given points.
function normalizeVector(x1, y1, x2, y2)
    local distance, a, b = distanceBetween(x1, y1, x2, y2)
    return a / distance, b / distance, distance
end

function distanceBetween(x1, y1, x2, y2)
    local a = x1 - x2
    local b = y1 - y2
    return math.sqrt((a*a) + (b*b)), a, b
end

function pointDirectlyUnderLine(pointX, pointY, lineX1, lineY1, lineX2, lineY2, bottomBound)
    -- This check currently assumes right-handedness.
    -- I.e. it assumes the ball is to the right of batBaseX
    if pointX < lineX1 or pointX > lineX2 or pointY > bottomBound then
        return false
    end

    local m = (lineY2 - lineY1) / (lineX2 - lineX1)

    -- y = mx + b
    -- b = y1 - (m * x1)
    local b = lineY1 - (m * lineX1)
    local yOnLine = (m * pointX) + b
    local yP = pointY
    local yDelta = yOnLine - yP

    return yDelta <= 0
end

blipper = {
    --- Build an object that simply "blips" between the given images at the given interval.
    --- Expects `playdate.graphics.animation.blinker.updateAll()` to be called on every update.
    new = function(msInterval, imagePath1, imagePath2)
        local blinker = playdate.graphics.animation.blinker.new(msInterval, msInterval, true)
        blinker:start()
        return {
            blinker = blinker,
            image1 = playdate.graphics.image.new(imagePath1),
            image2 = playdate.graphics.image.new(imagePath2),
            draw = function(self, disableBlipping, x, y)
                local currentImage = (disableBlipping or self.blinker.on) and self.image2 or self.image1
                currentImage:draw(x, y)
            end
        }
    end
}